/*
 * title:       userinfo
 *
 * author:      Craig E. Ward
 *
 * environment: MacOS X 10.2.5 (Darwin 6.5) GCC 3.1
 *
 * description:
 *              CMSI 587 Operating Systems, Spring 2003
 *              Assignment 8 -- Security and Protection & Distributed O/S
 *              Overview
 *              Instructor: Tri Nguyen
 *              Due: April 30
 *
 *              Program accepts a username and displays the information stored
 *              in the system security database.
 *
 */
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>

typedef enum {FALSE, TRUE, NO=0, YES} bool_t;

/* static function prototypes */
static void usage(void);
static bool_t showpwent(const char *user);

/*  external data  */
extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;
extern int optreset;

/* initialized data */

/* uninitialized data */
char *progname;

/*
 * Routine to display the Darwin/MacOS X password file entry for a username.
 *
 * NOTES:
 * (1) If the effective UID of the calling process is not 0, the string given
 *     in the pw_passwd field is the string "*". (This will also be the string
 *     for accounts that do not have a real login, e.g. mysql for the MySQL
 *     database.)
 * (2) Password aging is effectively not supported.
 * (3) "Access class" is a new field that is not used for anything as of this
 *     version of MacOS X/Darwin (10.2.5 and 6.5 respectively).
 */
static bool_t showpwent(const char *user)
{
  bool_t isOk = YES;
  struct passwd *pwe;           /* PassWord Entry */

  pwe = getpwnam(user);
  if (pwe == NULL) {
    fprintf(stderr, "%s: \"%s\" not found!\n", progname, user);
    isOk = NO;
  } else {
    printf("Data for %s\n", user);
    printf("\taccount name:\t\t%s\n", pwe->pw_name);
    printf("\tuser id:\t\t%d\n", pwe->pw_uid);
    printf("\tgroup id:\t\t%d\n", pwe->pw_gid);
    printf("\tpassword:\t\t%s\n", pwe->pw_passwd);
    printf("\tpassword expires:\t%s", ((pwe->pw_change == (time_t)0) ?
                            "never\n" :
                            ctime(&(pwe->pw_change))));
    printf("\taccess class:\t\t%s\n", ((*pwe->pw_class == NULL) ?
                             "unimplemented" :
                             pwe->pw_class));
    printf("\tpersonal data:\t\t%s\n", pwe->pw_gecos);
    printf("\thome directory:\t\t%s\n", pwe->pw_dir);
    printf("\tlogin shell:\t\t%s\n", pwe->pw_shell);
    printf("\taccount expires:\t%s", (pwe->pw_expire == (time_t)0 ?
                             "never\n" :
                             ctime(&(pwe->pw_expire))));
  }

  return isOk;
} /* showpwent */

int main(int argc, char *argv[])
{
  bool_t status = TRUE;
  char ch;
  char username[20];
  char password[12];

  progname = strrchr(argv[0], '/');
  if (progname == NULL) { progname = argv[0]; } else { progname++; }

  if (argc < 2)
    {
      usage();
      exit(EXIT_FAILURE);
    }

  memset(username, '\0', sizeof username);
  memset(password, '\0', sizeof password);
  while ((ch = getopt(argc, argv, "u:p:")) != -1) {
    switch (ch) {
    case 'u':
      strncpy(username, optarg, (sizeof username) - 1);
      break;

    case 'p':
      strncpy(password, optarg, (sizeof password) - 1);
      break;

    default:
      usage();
      exit(EXIT_FAILURE);
    }
  }

  if (username[0] == '\0') {
    fprintf(stderr,"%s: You must supply a username\n", progname);
    usage();
    status = FALSE;
  } else {
    showpwent(username);
  }

  if (status == TRUE)
    exit(EXIT_SUCCESS);
  exit(EXIT_FAILURE);
} /* main */

/*
 * Print a simple help message on standard error.
 */
static void usage(void)
{
  fprintf(stderr,"usage: %s -u username {-p password}\n", progname);
}
